<?php

define('WORKPLACE_PUT_MODE_NEWREVISION', 10, true);
define('WORKPLACE_PUT_MODE_REPLACE', 11, true);

define('WORKPLACE_ERROR_RUNTIME_PLUGIN_NOT_FOUND', 20, true);
define('WORKPLACE_ERROR_RUNTIME_INVALID_RESULT', 21, true);
define('WORKPLACE_ERROR_RUNTIME_REPLACE_FAILED', 22, true);
define('WORKPLACE_ERROR_RUNTIME_FILE_NOT_EXIST', 23, true);
define('WORKPLACE_ERROR_RUNTIME_FETCH_FAILED', 24, true);

// Initialize the configuration
require_once "O3Spaces/Config.php";
require_once "O3Spaces/RequestAuthStrategy.php";

// Folder filters
require_once "O3Spaces/Filter.php";

// HTTP Client
require_once "O3Spaces/HTTP/WP_HTTP_Request.php";

// WebDAV client
require_once "O3Spaces/HTTP/WebDAV/Client.php";

// SOAP client
require_once "O3Spaces/SOAP/WP_SOAP_Client.php";

// Utils
require_once "O3Spaces/Utils.php";


class WorkplaceIntegration {

	protected $cfg = NULL;
	protected $plugins = array();

	/** Constructor to initialize the service object
	 *
	 * @param config - A configuration object to use
	 */
	function __construct($config) {

		// Initialize the configuration object
		$this->cfg = Config::getConfig();
		$this->cfg->initialize($config);

	}

	/** Retrieve singleton instance of a plugin */
	public function getPluginInstance($name) {

		if (!isset($this->plugins[$name])) {
			if (! @include "O3Spaces/Plugins/$name.php") {
				return PEAR::raiseError("Cannot load module $name (File not found)", WORKPLACE_ERROR_RUNTIME_PLUGIN_NOT_FOUND);
			}
			$this->plugins[$name] = new $name($this);
		}

		return $this->plugins[$name];

	}

	/** Test if the configured authentication is suitable for the workplace */
	public function testAuthentication() {

		$response = $this->queryAuthentication();

		switch ($response) {
		case 200: // authorized
			return true;

		case 401: // unauthorized
			return false;

		default:
			return false;
		}

	}



	/** Query the contents of a webdav folder
	 *
	 * @param $path - The relative path to the file to download. E.g. Workspace%20One/Dir/File.jpg
	 * @param $filter - A BaseFilter object to filter only the files you want
	 * @return array - containing a list of "name", "webdav_path" and "type" array for each found file
	 */
	public function getFolderContents( $path, $filter=NULL ) {

		$entries = $this->queryWebDAVFolderContents($path);
		if ($filter != NULL) {
			$entries = $filter->filter($entries);
		}
		return $entries;

	}

	public function getFileInfo( $path ) {
		$urlPath = Utils::urlAppendPath($this->cfg->getWebDAVLocation(), $path);

		$x = false;
		if (false === $x = fopen($urlPath,"r")) {
			return PEAR::raiseError("File does not exist: $urlPath", WORKPLACE_ERROR_RUNTIME_INVALID_RESULT);
		}

		$streamObject = stream_get_meta_data($x);
		$streamObject = $streamObject["wrapper_data"];

		return $streamObject->getDirProps();
	}

	/** Download a file from the Workplace repository
	 *
	 * @param $path - The relative HTTP path to the file. E.g. Workspace%20One/Dir1/File.jpg
	 * @return path - Returns path to the temporary created file. Caller needs to clean up the file
	 */
	public function getFile( $path ) {
		$urlPath = Utils::urlAppendPath($this->cfg->getWebDAVLocation(), $path);

		if (!file_exists($urlPath)) {
			return PEAR::raiseError("File does not exist: $urlPath", WORKPLACE_ERROR_RUNTIME_FILE_NOT_EXIST);
		}

		$temp = tempnam( $this->cfg->get("store.tmp_dir"), "o3spaces-workplace-file" );
		if (false === $data = file_get_contents($urlPath)) {
			return PEAR::raiseError("Failed to retrieve data from workplace (path: $urlPath)", WORKPLACE_ERROR_RUNTIME_FETCH_FAILED);
		}

		if (file_put_contents($temp, $data)) {

			$info = $this->getFileInfo($path);
			if (PEAR::isError($info))
				$info = array();

			$info["tmpfile"] = $temp;
			return $info;

		} else
			return PEAR::raiseError("Could not write Workplace data to $temp", WORKPLACE_ERROR_RUNTIME_FETCH_FAILED);
	}

	/** Test if a file already exists on a location in the repository
	 *
	 * @param $name     - The plain filename (e.g. Sales Report 2009.xls)
	 * @param $location - The relative webdav-directory to put the file (e.g. Workspace%200ne/Sales%20Reports/)
	 *
	 * @returns
	 */
	public function testFileExists( $name, $location ) {

		$url = $this->createFilePathURL($location, $name);
		return file_exists($url);

	}

	/** Remote test to verify authentication configuration is correct */
	public function queryAuthentication() {

		$urlPath = Utils::urlAppendPath($this->cfg->getHTTPLocation(), "/authenticate");

		$r = &new WP_HTTP_Request($urlPath);
		$r->setMethod(HTTP_REQUEST_METHOD_GET);
		$r->sendRequest();
		$code = $r->getResponseCode();

		return $code;


	}

	/** Upload a local file to the Workplace
	 *
	 * Upload the file stored in $sourcePath to file $name in folder $destLocation in the workplace
	 *
	 * @param $name - The filename to use (e.g. "Sales Strategy 2009.xls")
	 * @param $sourcePath - The path to the local file to upload (e.g. /tmp/tmpXy134)
	 * @param $destLocation - Relative path to the destination webdav folder (e.g. Workspace%20One/Sales%20Strategy/)
	 * @param $mode - The mode to use when saving the file, can be WORKPLACE_PUT_MODE_NEWREVISION or WORKPLACE_PUT_MODE_REPLACE
	 * 
	 * @return bool - False if not successful. Otherwise true.
	 */
	public function putFile( $name, $sourcePath, $destLocation, $mode=WORKPLACE_PUT_MODE_NEWREVISION) {

		$urlPath = $this->createFilePathURL( $name, $destLocation );

		// Security check, path may not end with a / because that is a directory and can
		// potentially remove that directorytree completely
		if (strrpos($urlPath,"/") == (strlen($urlPath)-1)) {
			return PEAR::raiseError("Invalid path for a FILE: $urlPath", WORKPLACE_ERROR_RUNTIME_INVALID_RESULT);
		}

		// If replace is needed first delete the file, if it exists
		if ($mode == WORKPLACE_PUT_MODE_REPLACE && file_exists($urlPath) ) {
			if (false === unlink($urlPath)) {
				return PEAR::raiseError("Unlink failed. But going on", WORKPLACE_ERROR_RUNTIME_REPLACE_FAILED);
			}
		}

		if (false === copy($sourcePath, $urlPath)) {
			return PEAR::raiseError("Copy from $sourcePath to $urlPath failed", WORKPLACE_ERROR_RUNTIME_REPLACE_FAILED);
		}

		return true;

	}

	/** Query the contents of a webdav path
	 * 
	 * @param $path  - The relative path of the webfolder to query. Example: Workspace%201/Dir1/Dir2
	 * @return array - Containing "name", "webdav_path" and "type" fields pairs
	 */
	private function queryWebDAVFolderContents($path) {
		$urlPath = Utils::urlAppendPath($this->cfg->getWebDAVLocation(), $path);

		$files = array();

		if (is_dir($urlPath) && $dh = opendir($urlPath)) {
			$streamObject = stream_get_meta_data($dh);
			$streamObject = $streamObject["wrapper_data"];

			while (($file = readdir($dh)) !== false) {
				$file = array(
					"name" => urldecode($file),
					"webdav_path" => Utils::urlAppendPath($path, $file),
					"type" => filetype( Utils::urlAppendPath($urlPath, $file)),
					"props" => array()
				);

				// FIXME: why are files suddenly unknown??
				if ($file["type"] == "unknown")
					$file["type"] = "file";

				foreach ($streamObject->getProps() as $prop => $value) {
					$file["props"][$prop] = $value;
				}

				$files[] = $file;
			}
			closedir($dh);
		} else {
			PEAR::raiseError("Cannot open directory: $path on URL $urlPath");
		}

		return $files;

	}

	/** Create a new full webdav URL for $fileName in directory $location
	 *
	 * @param $filename - Plain, unencoded filename (e.g. Sales Strategy 2009.xls)
	 * @param $location - Relative webdav path to directory (e.g. Space%20One/Dir1/Cookies)
	 *
	 * @return url - Full webdav URL location for the file
	 */
	private function createFilePathURL( $filename, $location ) {
		$urlPath = Utils::urlAppendPath($this->cfg->getWebDAVLocation(), $location);
		$urlPath = Utils::urlAppendPath($urlPath, rawurlencode($filename));
		return $urlPath;
	}


};

?>
